@@ -2,12 +2,13 @@ import os
 import re
 import urllib
 import shutil
-from math import pi, cos, sin, log, exp, atan
 import logging
 import tempfile
 
 from mbutil import disk_to_mbtiles
 
+from proj import GoogleProjection
+
 has_mapnik = False
 try:
     import mapnik
@@ -29,57 +30,14 @@ DEFAULT_TILE_SIZE = 256
 """ Number of retries for remove tiles downloading """
 DOWNLOAD_RETRIES = 3
 
-DEG_TO_RAD = pi/180
-RAD_TO_DEG = 180/pi
-
 
 logger = logging.getLogger(__name__)
 
 
-def minmax (a,b,c):
-    a = max(a,b)
-    a = min(a,c)
-    return a
-
-
 class DownloadError(Exception):
     """ Raised when download at tiles URL fails DOWNLOAD_RETRIES times """
     pass
 
-class GoogleProjection(object):
-    """
-    Transform Lon/Lat to Pixel within tiles
-    Originally written by OSM team : http://svn.openstreetmap.org/applications/rendering/mapnik/generate_tiles.py     
-    """
-    def __init__(self, tilesize, levels = [0]):
-        self.Bc = []
-        self.Cc = []
-        self.zc = []
-        self.Ac = []
-        self.maxlevel = max(levels) + 1
-        c = tilesize
-        for d in range(self.maxlevel):
-            e = c/2;
-            self.Bc.append(c/360.0)
-            self.Cc.append(c/(2 * pi))
-            self.zc.append((e,e))
-            self.Ac.append(c)
-            c *= 2
-    
-    def fromLLtoPixel(self,ll,zoom):
-         d = self.zc[zoom]
-         e = round(d[0] + ll[0] * self.Bc[zoom])
-         f = minmax(sin(DEG_TO_RAD * ll[1]),-0.9999,0.9999)
-         g = round(d[1] + 0.5*log((1+f)/(1-f))*-self.Cc[zoom])
-         return (e,g)
-     
-    def fromPixelToLL(self,px,zoom):
-         e = self.zc[zoom]
-         f = (px[0] - e[0])/self.Bc[zoom]
-         g = (px[1] - e[1])/-self.Cc[zoom]
-         h = RAD_TO_DEG * ( 2 * atan(exp(g)) - 0.5 * pi)
-         return (f,h)
-
 
 class MBTilesBuilder(object):
     def __init__(self, **kwargs):
@@ -271,24 +229,32 @@ class MBTilesBuilder(object):
         """
         Render the specified tile with Mapnik
         """
-        if not self._mapnik:
-            self._mapnik = mapnik.Map(self.tile_size, self.tile_size)
-            # Load style XML
-            mapnik.load_map(self._mapnik, self.stylefile, True)
-            # Obtain <Map> projection
-            self._prj = mapnik.Projection(self._mapnik.srs)
-
         # Calculate pixel positions of bottom-left & top-right
         p0 = (x * self.tile_size, (y + 1) * self.tile_size)
         p1 = ((x + 1) * self.tile_size, y * self.tile_size)
-
         # Convert to LatLong (EPSG:4326)
-        l0 = self.proj.fromPixelToLL(p0, z);
-        l1 = self.proj.fromPixelToLL(p1, z);
+        l0 = self.proj.fromPixelToLL(p0, z)
+        l1 = self.proj.fromPixelToLL(p1, z)
+        return self.render(self.stylefile, 
+                           (l0[0], l0[1], l1[0], l1[1]), 
+                           output, 
+                           self.tile_size, self.tile_size)
+
+    def render(self, stylefile, bbox, output, width, height):
+        """
+        Render the specified bbox (minx, miny, maxx, maxy) with Mapnik
+        """
+        if not self._mapnik:
+            self._mapnik = mapnik.Map(width, height)
+            # Load style XML
+            mapnik.load_map(self._mapnik, stylefile, True)
+            # Obtain <Map> projection
+            self._prj = mapnik.Projection(self._mapnik.srs)
 
         # Convert to map projection
-        c0 = self._prj.forward(mapnik.Coord(l0[0],l0[1]))
-        c1 = self._prj.forward(mapnik.Coord(l1[0],l1[1]))
+        assert len(bbox) == 4, "Provide a bounding box tuple (minx, miny, maxx, maxy)"
+        c0 = self._prj.forward(mapnik.Coord(bbox[0],bbox[1]))
+        c1 = self._prj.forward(mapnik.Coord(bbox[2],bbox[3]))
 
         # Bounding box for the tile
         if hasattr(mapnik,'mapnik_version') and mapnik.mapnik_version() >= 800:
@@ -296,11 +262,11 @@ class MBTilesBuilder(object):
         else:
             bbox = mapnik.Envelope(c0.x,c0.y, c1.x,c1.y)
         
-        self._mapnik.resize(self.tile_size, self.tile_size)
+        self._mapnik.resize(width, height)
         self._mapnik.zoom_to_box(bbox)
         self._mapnik.buffer_size = 128
 
         # Render image with default Agg renderer
-        im = mapnik.Image(self.tile_size, self.tile_size)
+        im = mapnik.Image(width, height)
         mapnik.render(self._mapnik, im)
         im.save(output, 'png256')
